
#include "stdafx.h"

#include "sortdef.h"

/////////////////// General support //////////////////////////////////
IMPLEMENT_SERIAL( MSortDef, MFlatFile, 1 ) 

////////////////////////////////////////////////////////////////////////
// Serialize - Standard MFC routine to Serialize the MSortDef object
void MSortDef::Serialize( CArchive & ar )
{
	// Call the Serialize routines in the Co
	MFlatFile::Serialize( ar );
	SortArray.Serialize( ar );
	OutputArray.Serialize( ar );
	
	if( ar.IsStoring() )
		ar << OutputName << KeySize << RecSep << Ascending;
	else
		ar >> OutputName >> KeySize >> RecSep >> Ascending;
}

/////////////////////////////////////////////////////////////////
//  int MSortDef::FindSort( const char *Name )
//  Descr: Finds a sort definition in the current list by fieldname
//  Parameters: Name - Name of field to search for
//  Returns: -1 if not found, otherwise index position
//
int MSortDef::FindSort( const char *Name )
{
	int i;
	for( i=0; i < GetSortCount(); i++ )
	{
		if( GetSort(i)->IsName(Name) )
			return(i);
	}
	return(-1);
}

/////////////////////////////////////////////////////////////////
//  int MSortDef::FindOutput( const char *Name )
//  Descr: Finds an output definition in the current list by fieldname
//  Parameters: Name - Name of field to search for
//  Returns: -1 if not found, otherwise index position
//
int MSortDef::FindOutput( const char *Name )
{
	int i;
	for( i=0; i < GetOutputCount(); i++ )
	{
		if( GetOutput(i)->IsName(Name) )
			return(i);
	}
	return(-1);
}

/////////////////////////////////////////////////////////////////
//  int MSortDef::Reset()
//  Descr: Initializes the MSortDef, for re-use
//
void MSortDef::Reset()
{
	int i;
	for( i=0; i<SortArray.GetSize(); i++ )
		delete SortArray[i];
	SortArray.RemoveAll();
	Sorted.RemoveAll();

	for( i=0; i<OutputArray.GetSize(); i++ )
		delete OutputArray[i];
	OutputArray.RemoveAll();
	OutputName="";
	KeySize=0;
	SetRecSep(0);
	SetAscending(1);
	// Note, because Close is a virtual function, the Reset() code
	// actually calls it for us.  But, this is sort of an 'esoteric' fact,
	// and I left the code (Close call) here for clarity.
	MFlatFile::Reset();
}



////////////////////// Sort list support ///////////////////////////////

/////////////////////////////////////////////////////////////////
//  void MSortDef::AddSort( CString Name, BYTE Sensitive )
//  Descr: Adds a sort definition to the MSortDef object
//  Parameters: Name - Name of field for new sort definition
//              Sensitive
//  Returns: -1 if not found, otherwise index position
//
void MSortDef::AddSort( CString Name, BOOL Sensitive )
{
	InsertSort( Name, Sensitive, GetSortCount() );
}

void MSortDef::InsertSort( CString Name, BOOL Sensitive, int i )
{
	SortArray.InsertAt( i, new MSort( Name, Sensitive ));
	KeySize +=  GetField(GetSort(i)->GetFieldName())->GetSize();
}

void MSortDef::SetSort( CString Name, BOOL Sensitive, int i )
{
	KeySize -=  GetField(GetSort(i)->GetFieldName())->GetSize();
	((MSort*)SortArray[i])->Set(Name, Sensitive );
	KeySize +=  GetField(GetSort(i)->GetFieldName())->GetSize();
	
}

void MSortDef::RemoveSort( int i )
{ 	
	KeySize -=  GetField(GetSort(i)->GetFieldName())->GetSize();
	delete GetSort(i); 
	SortArray.RemoveAt(i); 
}

	
CString MSortDef::GetSortDescr( int i )
{
	CString Descr;

	if( i <0 || i> GetSortCount() )
		Descr="";
	else
	{
		Descr = GetSortName(i) + ", ";
		Descr += (IsSensitive(i)?"Sens":"Insens");
	}
	return( Descr );
	
}

//////////////////////  File support ////////////////////////////////
//

void MSortDef::CloseOutputFile()
{
	Output.Close();
}
BOOL MSortDef::CreateOutputFile( )
{
	return (Output.Open( OutputName, CFile::modeReadWrite|CFile::modeCreate ) );
}


/////////////////// Output maintenance support /////////////////////////

/////////////////////////////////////////////////////////////////
// Appends an output definition
void MSortDef::AddOutput( CString Name, CString Postfix, WORD Options )
{
	InsertOutput(  Name, Postfix, Options , GetOutputCount() );
}

/////////////////////////////////////////////////////////////////
// Inserts an output definition
void MSortDef::InsertOutput(  CString Name, CString Postfix, WORD Options, int i )
{
	OutputArray.InsertAt( i, new MOutput( Name, Postfix, Options  ));
}

/////////////////////////////////////////////////////////////////
// Updates an output definition
void MSortDef::SetOutput(  CString Name, CString Postfix, WORD Options, int i )
{
	((MOutput*)OutputArray[i])->Set( Name, Postfix, Options  );
}

/////////////////////////////////////////////////////////////////
// Returns text description of an output definition
CString MSortDef::GetOutputDescr( int i )
{
	CString Descr;

	if( i <0 || i> GetOutputCount() )
		Descr="";
	else
	{
		Descr = GetOutputName(i) + ", ";
		Descr += IsEndLine(i)?"EOL,":"Not EOL,";
		Descr += IsTrim(i)?"Trim, ": "No Trim, ";
		Descr += HasSep(i)?"Sep, ":"No Sep";
		Descr += AllowBlank(i)?"Blank OK":"No Blank";
	}
	return( Descr );
}

/////////////////////////////////////////////////////////////////
// Builds a key for the current record, and adds it to the sorted list
void MSortDef::AddRecordSort(WORD Rec)
{
	CString Tmp, Key;
	int i;
	unsigned int s;

	for( i=0; i < GetSortCount(); i++ )
	{	
		Tmp =   GetField(GetSortName(i))->GetAsString();
		if( !IsSensitive(i) )
			Tmp.MakeUpper();
		Key += Tmp;
		s = GetField(GetSortName(i))->GetSize();
		while( strlen(Key) < s )
			Key += " ";

	}
	Sorted.Add( Key, Rec );
}

/////////////////////////////////////////////////////////////////
// Outputs a record, using the Output list for contents and formatting 
// information
void MSortDef::OutputRecord()
{
	int i, HasAnyData=0;
	MOutput * Out;
	CString Line, Tmp;

	Line="";
	for(i=0; i < GetOutputCount(); i++ )
	{
		Tmp = GetField(GetOutputName(i))->GetAsString();
		Out = GetOutput(i);
		if( Out->IsTrim() )
			Tmp.TrimRight();
		if( strlen(Tmp) )
		{
			Tmp += Out->GetPostfix();
			if( Out->HasSep() )
				Tmp += " ";
		}
		Line += Tmp;
		if( Out->IsEndLine() )
		{
			if( strlen(Line) || Out->AllowBlank() )
			{
				Output.Write( Line, strlen(Line) );
				Output.Write("\r\n", strlen("\r\n") );
				HasAnyData=1;
			}
			Line="";
		}

	}
	if( Line!="" )
	{			
		Output.Write( Line, strlen(Line) );
		HasAnyData=1;
	}

	if( HasAnyData && HasRecSep() )
	{
		Output.Write( Line, strlen(Line) );
		Output.Write("\r\n", strlen("\r\n") );
	}


}

/////////////////////////////////////////////////////////////////
// Performs the sort
BOOL MSortDef::DoSort( CWnd * ToUpdate, CString & Result )
{
	WORD Rec;
	BOOL Ret=TRUE;
	TRY
	{
		if( Open(CFile::modeRead) ==FALSE )
		{
			Result = "Error opening input file";
			return(FALSE);
		}
		if( Output.Open( OutputName, CFile::modeCreate|CFile::modeRead|CFile::modeWrite)==FALSE )
		{
			Close();
			Result = "Error opening output file";
			return(FALSE);
		}

		Sorted.SetAscending( Ascending );
		
		// Sort the list, by adding a record sort key for each
		for( Rec = 0; Rec < GetRecCount(); Rec++ )
		{
			Goto( Rec );
			ReadRecord();
			AddRecordSort(Rec);
		}

		// Output the list, paying attention to the format.
		for( Rec = 0; Rec < GetRecCount(); Rec++ )
		{
			Goto( GetSortedRec(Rec) );
			ReadRecord();
			OutputRecord();
		}

		Output.Close();
		CFile::Close();

		Result = "Records sorted: ";
		char Tmp[16];
		wsprintf( Tmp, "%d", Rec );
		// Comment out for a test: ThrowFieldException( "A test", "Bogus");
		Result += Tmp;
	}
	CATCH_ALL(E)
	{
		E->GetErrorMessage( Result.GetBuffer(128), 128, NULL );
		Result.ReleaseBuffer();
		Ret = FALSE;
	}
	END_CATCH_ALL

	Sorted.RemoveAll();
	return( Ret );

}
